/***************************************************************************
 *       GOnOffMakerReflected.hpp - Class for ON-OFF data preparation      *
 * ----------------------------------------------------------------------- *
 *  copyright (C) 2009-2013 by Juergen Knoedlseder                         *
 * ----------------------------------------------------------------------- *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 *                                                                         *
 ***************************************************************************/
/**
 * @file GOnOffMakerReflected.hpp
 * @brief Class interface for ON-OFF data preparation from reflected regions
 * @author Pierrick Martin
 */

#ifndef GONOFFMAKERREFLECTED_HPP
#define GONOFFMAKERREFLECTED_HPP

/* __ Includes ___________________________________________________________ */
#include <vector>
#include <string>

/* __ Forward declarations _______________________________________________ */
class GObservations;
class GEbounds;
class GSkyRegion;

/***********************************************************************//**
* @class GOnOffMakerReflected
*
* @brief Class interface for ON-OFF data preparation from reflected regions
*
* This class defines the interface for the preparation of ON and OFF data 
* using the so-called reflected region approach (see Berge et al. 2007).
* The class is derived from the abstract class GOnOffMaker.
*
* For a complete setup, the object needs:
* - An observations container
* - Definition of energy grid through GEbounds object
* - Definition of the ON region through a GSkyRegion object
* - The number of OFF regions
* The OFF regions will be evenly distributed about the centre of the 
* field-of-view in each observation run, at the same off-axis angular 
* distance as the ON region. More sophisticated schemes can/will be 
* implemented later.
*
* The method compute_data() loops over all runs in the observation 
* container and all energy bins and fills the GOnOffBin array based on 
* the internal methods sum_counts() and sum_acceptance() (the latter being 
* useless in the reflected region approach).
*
**************************************************************************/
class GOnOffMakerReflected : public GOnOffMaker {
	
public:
    // Constructors and destructors
    GOnOffMakerReflected(void);
    GOnOffMakerReflected(const GOnOffMakerReflected& maker);
	explicit GOnOffMakerReflected(const GObservations& obs, const GEbounds& ebds, const GSkyRegion& on, const int numoff);
	explicit GOnOffMakerReflected(const std::string& filename, const GEbounds& ebds, const GSkyRegion& on, const int numoff);
    virtual ~GOnOffMakerReflected(void);
	
    // Operators
    virtual GOnOffMakerReflected&  operator=(const GOnOffMakerReflected& maker);
  	
    // Implemented pure virtual base class methods
    virtual void                   clear(void);
    virtual GOnOffMakerReflected*  clone(void) const;
    virtual std::string            type(void) const;
    virtual std::string            print(const GChatter& chatter = NORMAL) const;
	
    // Methods specific to the derived class
	void                           ebounds(const GEbounds& ebds);
	const GEbounds*                ebounds(void) const;	
	void                           onregion(const GSkyRegion& on);
	const GSkyRegion*              onregion(void) const;
	void                           numoff(const int numoff);
	const int                      numoff(void) const;	
	void                           compute_data(void);
	
protected:
    // Protected methods
    void         init_members(void);
    void         copy_members(const GOnOffMakerReflected& maker);
    void         free_members(void);
	void         sum_counts(GObservation& obs, GSkyRegion& reg);
	void         sum_acceptance(GObservation& obs, GSkyRegion& reg);
	
    // Protected members
	GEbounds*           m_ebds;         //!< Energy binning definition
	GSkyRegion*         m_regon;        //!< ON sky region
	int                 m_numoff;       //!< Number of OFF regions
};

#endif /* GONOFFMAKERREFLECTED_HPP */

																		  
																
																		
															
